import { Board } from "./entities/Board";
import { PlayerInterface } from "./entities/player/PlayerInterface";
import { askQuestion, printError, printSelection, printTiles, wait } from "../common/utilities";
import { PlayerMove } from "./entities/PlayerMove";
import { Tile } from "./entities/Tile";
import { PlayerMoveSide, PlayerMoveSideType } from "./constants";
import { PlayerInteractionInterface } from "./PlayerInteractionInterface";
import { InteractionService } from "../server/services/InteractionService";

export class PlayerInteractionConsole implements PlayerInteractionInterface {
  player: PlayerInterface;
  interactionService: InteractionService = new InteractionService();
  
  constructor(player: PlayerInterface) {
    this.player = player;
  }

  askForMove(board: Board): void {
    wait(100)
  }

  async makeMove(board: Board): Promise<PlayerMove | null> {
    let move: PlayerMove | null = null;
    let tile: Tile;
    let side: PlayerMoveSideType | null = null;
    const { player } = this;

    printSelection('Hand: ', player.hand);
    while (move === null) {
      const answer = await askQuestion('Enter your move (tile index side is L or R, e.g. 0L, <Enter> to pass');
      const char0 = answer.charAt(0);
      const char1 = answer.charAt(1);
      if (answer === '' || answer === '') {
        return null;
      }
    
      if (char1 === 'L' || char1 === 'R' || char1 === 'l' || char1 === 'r') {
        side = char1 === 'L' || char1 === 'l' ? PlayerMoveSide.LEFT : PlayerMoveSide.RIGHT;
      }
    
      if (char0 > '0' && char0 <= String(player.hand.length)) {
        const tileIndex = parseInt(char0) - 1;
        tile = player.hand[tileIndex];
        move = board.isValidMove(tile, side, player);
        if (move === null) {
          printError('Invalid move');
        }
      }
    }
    return move;
  }

  async chooseTile(board: Board): Promise<Tile> {
    const { player: { hand} } = this;
    let index: number = -1;
    while (index < 0 || index >= board.boneyard.length) {
      printTiles('Hand: ', hand);
      printSelection('Boneyard: ', board.boneyard);
      const answer = await askQuestion('Choose a tile from the boneyard');
      if (answer === '') {
        index = 0;
        continue;
      }
      if (answer < '0' || answer > '9') {
        printError('Invalid selection');
        continue;
      }
      index = parseInt(answer) - 1;
    }    
    const tile = board.boneyard.splice(index, 1)[0];
    tile.revealed = true;
    hand.push(tile);
    return tile;
  }
}

