import { GameSummary } from "../../game/dto/GameSummary";
import { MatchSessionOptions } from "../../game/dto/MatchSessionOptions";
import { PlayerDto } from "../../game/dto/PlayerDto";

export interface Entity {
  createdAt?: number;
  modifiedAt?: number;
  createdBy?: string;
  modifiedBy?: string;
}

export interface EntityMongo extends Entity {
  _id?: string;
}

export interface Score {
  id: string;
  name: string;
  score: number;
}

export interface Namespace extends EntityMongo {
  name?: string;
  description?: string;
  default: boolean;
  type: string | null;
  ownerId?: string;
  users?: any[];
}

export interface User extends EntityMongo {
  id: string,
  username: string;
  namespaceId: string;
  hash?: string;
  roles: string[];
  firstname?: string;
  lastname?: string;
  email?: string;
  profileId?: string;
  password?: string | null;
  namespace?: Namespace;
}

export interface DbMatchSessionUpdate extends EntityMongo {
  state?: string;
}

export interface DbMatchSession extends EntityMongo {
  id: string;
  name: string;
  creator: string;
  players: PlayerDto[];
  seed: string;
  mode: string;
  options: MatchSessionOptions;
  maxPlayers: number;
  numPlayers: number;
  scoreboard: Score[];
  matchWinner: PlayerDto | null;
  status: string;
  matchInProgress: boolean;
  gameSummaries: GameSummary[];
  room: string;
}

export interface DbUser extends EntityMongo {
  id: string,
  username: string;
  namespaceId: string;
  hash?: string;
  roles: string[];
  firstname?: string;
  lastname?: string;
  email?: string;
  profileId?: string;
  password?: string | null;
  namespace?: DbNamespace;
}

export interface DbNamespace extends EntityMongo {
  name?: string;
  description?: string;
  default: boolean;
  type: string | null;
  ownerId?: string;
}

export interface Token extends EntityMongo {
  token: string;
  userId?: string;
  roles?: string[];
  expiresAt?: number | null;
  type: string;
  namespaceId?: string
  description?: string;
}

export interface AuthenticationOption {
  roles?: string[];
}

export interface Role {
  name: string;
  description: string;
  permissions: string[];
}

export interface DbListResponse{
  pagination?: {
    page: number;
    next?: number;
    size: number;
    total: number;
    totalPages: number;
  }
  sort?: any;
  data: EntityMongo[];
}

export interface UserSession extends EntityMongo {
  userId: string;
  uuid: string;
}