import { uuid } from "../../../common/utilities";
import { UserSession } from "../interfaces";
import { BaseMongoManager } from "./common/BaseMongoManager";

export class UserSessionMongoManager extends BaseMongoManager {
  collection = 'userSessions';
  constructor() {
    super();
  }

  async createSession(userId: string): Promise<string> {
    const uid: string = uuid();
    const userSession: UserSession = {
      userId,
      uuid: uid
    }
    await this.create(userSession);
    return uid;
  }

  async removeSession(userId: string): Promise<number> {
    return this.deleteByFilter({ userId });
  }

  async renewSession(userId: string): Promise<string> {
    await this.removeSession(userId);
    return this.createSession(userId);
  }

  async getUserSession(userId: string): Promise<UserSession | null> {
    return this.getByFilter({ userId }) as Promise<UserSession | null>;
  }

  async getByUuid(uuid: string): Promise<UserSession | null> {
    return this.getByFilter({ uuid }) as Promise<UserSession | null>;
  }
}
