import { ObjectId } from "mongodb";

export class PipelineLibrary {
  static usersGetById(id: ObjectId) {
    return [
      {
        '$match': {
          '_id': id
        }
      }, {
        '$lookup': {
          'from': 'namespaces', 
          'localField': 'namespaceId', 
          'foreignField': '_id', 
          'as': 'namespace'
        }
      }, {
        '$unwind': {
          'path': '$namespace', 
          'preserveNullAndEmptyArrays': true
        }
      }
    ];
  }
  static namespacesGetNamespaces() {
    return [
      {
        '$lookup': {
          'from': 'users', 
          'localField': '_id', 
          'foreignField': 'namespaceId', 
          'as': 'users'
        }
      }, {
        '$project': {
          '_id': 1, 
          'name': 1, 
          'description': 1, 
          'ownerId': 1, 
          'default': 1, 
          'createdAt': 1, 
          'modifiedAt': 1, 
          'users': {
            '_id': 1, 
            'id': 1, 
            'username': 1, 
            'firstname': 1, 
            'lastname': 1, 
            'email': 1, 
            'profileId': 1
          }
        }
      }
    ];
  }

  static namespacesGetById(id: ObjectId) {
    return [
      {
        '$match': {
          '_id': id
        }
      },
      ... PipelineLibrary.namespacesGetNamespaces()
    ];
  }

  static usersGetAvailableUsers(defaultId: Object) {
    return [
      {
        '$match': {
          'namespaceId': defaultId
        },
      },
      ...PipelineLibrary.usersGetUsers(),
    ];
  }

  static usersGetUsers() {
    return [
      {
        '$lookup': {
          'from': 'namespaces', 
          'localField': 'namespaceId', 
          'foreignField': '_id', 
          'as': 'namespace'
        }
      }, {
        '$unwind': {
          'path': '$namespace',
          'preserveNullAndEmptyArrays': true
        }
      }, {
        '$project': {
          '_id': 1, 
          'username': 1, 
          'roles': 1, 
          'firstname': 1, 
          'lastname': 1, 
          'email': 1, 
          'modifiedAt': 1, 
          'createdAt': 1, 
          'namespace': {
            '_id': 1, 
            'name': 1
          }
        }
      }
    ];
  }
}