import express from 'express';
import http from 'http';
import cors from 'cors';
import { join } from 'path';

import { NetworkClientNotifier } from '../game/NetworkClientNotifier';
import { SocketIoService } from './services/SocketIoService';
import { LoggingService } from '../common/LoggingService';
import { useRouter } from './router';

const clientNotifier = new NetworkClientNotifier();
const logger = new LoggingService();
const app = express();
const httpServer = http.createServer(app);
const socketIoService = new SocketIoService(httpServer);
clientNotifier.setSocket(socketIoService.getServer());

const PORT = process.env.PORT || 3000;

app.use(cors());
app.use(logger.middleware());
app.use(express.json({ limit: '50mb'}));
app.use(express.text());
app.use(express.urlencoded({extended: true }));
app.use(useRouter())

app.use(express.static(join(process.cwd(), 'public')));

app.get('/', (req, res) => {
  res.sendFile(join(__dirname, 'index.html'));
});

httpServer.listen(PORT, () => {
  logger.info(`listening on *:${PORT}`);
});
