import crypto from 'crypto';
import jwt from 'jsonwebtoken';
import bcrypt from 'bcryptjs';
import { User } from '../db/interfaces';
import { ManagerBase } from './ManagerBase';

export class SecurityManager extends ManagerBase {
  saltRounds = Number(process.env.SALT_ROUNDS);
  jwtSecretKey = process.env.JWT_SECRET_KEY || '';

  generateId() {
    return crypto.randomBytes(16).toString('hex');
  }  

  getHashedPassword(password: string) {
    const salt = bcrypt.genSaltSync(this.saltRounds);
    return bcrypt.hashSync(password, salt);
  }

  generateApiToken() {
    return crypto.randomBytes(32).toString('hex');
  }

  signJwt(data: any, longTerm: boolean = false): string {
    const expiresIn: string = longTerm ? '7d' : '3h'
    delete data.iat;
    delete data.exp;
    return jwt.sign(data, this.jwtSecretKey, { expiresIn });
  }

  // TODO: verificar esto
  async verifyJwt(token: string): Promise<any> {
    return new Promise((resolve, reject) => {
      jwt.verify(token, this.jwtSecretKey, (err, decoded) => {
        if (err) {
          reject(err);
        } else {
          resolve(decoded);
        }
      });
    });
  }
}