import { DominoesGame } from "../../game/DominoesGame";
import { MatchSession } from "../../game/MatchSession";
import { NetworkClientNotifier } from "../../game/NetworkClientNotifier";
import { GameState } from "../../game/dto/GameState";
import { NetworkPlayer } from "../../game/entities/player/NetworkPlayer";
import { PlayerInterface } from "../../game/entities/player/PlayerInterface";
import { ServiceBase } from "./ServiceBase";

export class PlayerNotificationService extends ServiceBase {
  clientNotifier: NetworkClientNotifier = new NetworkClientNotifier();

  notifyGameState(game: DominoesGame) {
    const gameState: GameState = game.getState();
    const { players } = game;
    players.filter(p => p instanceof NetworkPlayer).map(player => player.sendEvent('update-game-state', gameState));
  }

  notifyPlayersState(players: PlayerInterface[]) {
    players.filter(p => p instanceof NetworkPlayer).map(player => player.sendEvent('update-player-state', player.getState()));
  }


  notifyMatchState(session: MatchSession) {
    const { players } = session;
    players.filter(p => p instanceof NetworkPlayer).forEach(player => player.sendEvent('update-match-session-state', session.getState()));
  }

  async sendEventToPlayers(event: string, players: PlayerInterface[], data: Function | any = {}) {
    players.filter(p => p instanceof NetworkPlayer).forEach((player) => {
      let dataTosend = data;
      if (typeof data === 'function') {
        dataTosend = data(player);
      }
      this.clientNotifier.sendEvent(player as NetworkPlayer, event, dataTosend);      
    });
  }

  sendEvent(event: string, player: PlayerInterface, data: any = {}) {  
    this.logger.debug(`Sending event '${event}' to player ${player.id}`);
    if (player instanceof NetworkPlayer) {
      this.clientNotifier.sendEvent(player as NetworkPlayer, event, data);  
    }
  }
}